/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.config;

import net.sf.intelliplugin.visualgc.util.JDOMExternalizer;
import org.jdom.Element;

import java.awt.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;
import java.util.Map;

/**
 * The singleton configuration instance.
 * 
 * @author Bart Cremers
 * @since 3-dec-2007
 */
@SuppressWarnings ({ "deprecation" })
public class IntelliHeapConfiguration {

    public static final String BORDER_FONT = "border.font";

    public static final String BORDER_COLOR = "border.color";
    public static final String BACKGROUND_COLOR = "background.color";

    public static final String SURVIVOR_COLOR = "survivor.color";
    public static final String EDEN_COLOR = "eden.color";
    public static final String TENURED_COLOR = "tenured.color";
    public static final String PERM_COLOR = "perm.color";
    public static final String HOTSPOT_COLOR = "hotspot.color";

    public static final String LOADER_COLOR = "loader.color";
    public static final String GC_COLOR = "gc.color";
    public static final String FINALIZER_COLOR = "finalizer.color";

    public static final String PRIMARY_GRID_COLOR = "primary.grid.color";
    public static final String SECUNDARY_GRID_COLOR = "secundary.grid.color";

    private static final Font DEFAULT_BORDER_FONT = new Font("Dialog", Font.BOLD, 12);

    private static final Color DEFAULT_BORDER_COLOR = Color.DARK_GRAY;
    private static final Color DEFAULT_BACKGROUND_COLOR = new Color(242, 242, 242);

    private static final Color DEFAULT_SURVIVOR_COLOR = new Color(49, 206, 99);
    private static final Color DEFAULT_EDEN_COLOR = new Color(0, 173, 0);
    private static final Color DEFAULT_TENURED_COLOR = new Color(0, 149, 215);
    private static final Color DEFAULT_PERM_COLOR = new Color(181, 181, 85);
    private static final Color DEFAULT_HOTSPOT_COLOR = new Color(161, 50, 50);

    private static final Color DEFAULT_LOADER_COLOR = new Color(241, 151, 60);
    private static final Color DEFAULT_GC_COLOR = new Color(201, 80, 181);
    private static final Color DEFAULT_FINALIZER_COLOR = Color.BLACK;

    private static final Color DEFAULT_PRIMARY_GRID_COLOR = Color.LIGHT_GRAY;
    private static final Color DEFAULT_SECUNDARY_GRID_COLOR = DEFAULT_PRIMARY_GRID_COLOR.darker().darker();

    private static final IntelliHeapConfiguration configuration = new IntelliHeapConfiguration();

    private final Map<String, Object> valueMap = new HashMap<String, Object>();
    private final Map<String, Object> uncommitedValueMap = new HashMap<String, Object>();

    private ConfigurationPanel configurationPanel;
    private final PropertyChangeSupport changeSupport;

    private IntelliHeapConfiguration() {
        valueMap.put(BORDER_FONT, DEFAULT_BORDER_FONT);

        valueMap.put(BORDER_COLOR, DEFAULT_BORDER_COLOR);
        valueMap.put(BACKGROUND_COLOR, DEFAULT_BACKGROUND_COLOR);

        valueMap.put(SURVIVOR_COLOR, DEFAULT_SURVIVOR_COLOR);
        valueMap.put(EDEN_COLOR, DEFAULT_EDEN_COLOR);
        valueMap.put(TENURED_COLOR, DEFAULT_TENURED_COLOR);
        valueMap.put(PERM_COLOR, DEFAULT_PERM_COLOR);
        valueMap.put(HOTSPOT_COLOR, DEFAULT_HOTSPOT_COLOR);

        valueMap.put(LOADER_COLOR, DEFAULT_LOADER_COLOR);
        valueMap.put(GC_COLOR, DEFAULT_GC_COLOR);
        valueMap.put(FINALIZER_COLOR, DEFAULT_FINALIZER_COLOR);

        valueMap.put(PRIMARY_GRID_COLOR, DEFAULT_PRIMARY_GRID_COLOR);
        valueMap.put(SECUNDARY_GRID_COLOR, DEFAULT_SECUNDARY_GRID_COLOR);

        changeSupport = new PropertyChangeSupport(this);
    }

    public static IntelliHeapConfiguration getConfiguration() {
        return configuration;
    }

    @SuppressWarnings ({ "unchecked", "UnusedDeclaration", "UnusedParameters" })
    private <T> T getValue(Class<T> clazz, String key) {
        return (T) valueMap.get(key);
    }

    void setValue(String key, Object value) {
        if (uncommitedValueMap.containsKey(key)) {
            uncommitedValueMap.remove(key);
        }

        if (!value.equals(valueMap.get(key))) {
            uncommitedValueMap.put(key, value);
        }
    }

    Color getColor(String key) {
        return getValue(Color.class, key);
    }

    Font getFont(String key) {
        return getValue(Font.class, key);
    }

    public Font getBorderFont() {
        return getFont(BORDER_FONT);
    }

    public Color getBorderColor() {
        return getColor(BORDER_COLOR);
    }

    public Color getBackgroundColor() {
        return getColor(BACKGROUND_COLOR);
    }

    public Color getSurvivorColor() {
        return getColor(SURVIVOR_COLOR);
    }

    public Color getEdenColor() {
        return getColor(EDEN_COLOR);
    }

    public Color getTenuredColor() {
        return getColor(TENURED_COLOR);
    }

    public Color getPermColor() {
        return getColor(PERM_COLOR);
    }

    public Color getHotspotColor() {
        return getColor(HOTSPOT_COLOR);
    }

    public Color getLoaderColor() {
        return getColor(LOADER_COLOR);
    }

    public Color getGcColor() {
        return getColor(GC_COLOR);
    }

    public Color getFinalizerColor() {
        return getColor(FINALIZER_COLOR);
    }

    public Color getPrimaryGridColor() {
        return getColor(PRIMARY_GRID_COLOR);
    }

    public Color getSecundaryGridColor() {
        return getColor(SECUNDARY_GRID_COLOR);
    }

    public ConfigurationPanel getConfigurationPanel() {
        if (configurationPanel == null) {
            synchronized (this) {
                configurationPanel = new ConfigurationPanel(this);
            }
        }
        return configurationPanel;
    }

    public boolean isModified() {
        return !uncommitedValueMap.isEmpty();
    }

    public void apply() {
        for (Map.Entry<String, Object> entry : uncommitedValueMap.entrySet()) {
            Object oldValue = valueMap.put(entry.getKey(), entry.getValue());
            changeSupport.firePropertyChange(entry.getKey(), oldValue, entry.getValue());
        }
        uncommitedValueMap.clear();
    }

    public void reset() {
        uncommitedValueMap.clear();
    }

    public void readExternal(Element root) {
        setValue(BORDER_COLOR, JDOMExternalizer.readColor(root, BORDER_COLOR, getBorderColor()));
        setValue(BACKGROUND_COLOR, JDOMExternalizer.readColor(root, BACKGROUND_COLOR, getBackgroundColor()));

        setValue(SURVIVOR_COLOR, JDOMExternalizer.readColor(root, SURVIVOR_COLOR, getSurvivorColor()));
        setValue(EDEN_COLOR, JDOMExternalizer.readColor(root, EDEN_COLOR, getEdenColor()));
        setValue(TENURED_COLOR, JDOMExternalizer.readColor(root, TENURED_COLOR, getTenuredColor()));
        setValue(PERM_COLOR, JDOMExternalizer.readColor(root, PERM_COLOR, getPermColor()));
        setValue(HOTSPOT_COLOR, JDOMExternalizer.readColor(root, HOTSPOT_COLOR, getHotspotColor()));

        setValue(LOADER_COLOR, JDOMExternalizer.readColor(root, LOADER_COLOR, getLoaderColor()));
        setValue(GC_COLOR, JDOMExternalizer.readColor(root, GC_COLOR, getGcColor()));
        setValue(FINALIZER_COLOR, JDOMExternalizer.readColor(root, FINALIZER_COLOR, getFinalizerColor()));

        setValue(PRIMARY_GRID_COLOR, JDOMExternalizer.readColor(root, PRIMARY_GRID_COLOR, getPrimaryGridColor()));
        setValue(SECUNDARY_GRID_COLOR, JDOMExternalizer.readColor(root, SECUNDARY_GRID_COLOR, getSecundaryGridColor()));

        apply();
    }

    public void writeExternal(Element root) {
        JDOMExternalizer.write(root, BORDER_COLOR, getBorderColor());
        JDOMExternalizer.write(root, BACKGROUND_COLOR, getBackgroundColor());

        JDOMExternalizer.write(root, SURVIVOR_COLOR, getSurvivorColor());
        JDOMExternalizer.write(root, EDEN_COLOR, getEdenColor());
        JDOMExternalizer.write(root, TENURED_COLOR, getTenuredColor());
        JDOMExternalizer.write(root, PERM_COLOR, getPermColor());
        JDOMExternalizer.write(root, HOTSPOT_COLOR, getHotspotColor());

        JDOMExternalizer.write(root, LOADER_COLOR, getLoaderColor());
        JDOMExternalizer.write(root, GC_COLOR, getGcColor());
        JDOMExternalizer.write(root, FINALIZER_COLOR, getFinalizerColor());

        JDOMExternalizer.write(root, PRIMARY_GRID_COLOR, getPrimaryGridColor());
        JDOMExternalizer.write(root, SECUNDARY_GRID_COLOR, getSecundaryGridColor());
    }

    public void addPropertyChangeListener(PropertyChangeListener l) {
        changeSupport.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l) {
        changeSupport.removePropertyChangeListener(l);
    }
}
