/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.model;

import net.sf.intelliplugin.visualgc.util.Converter;

/**
 * A GCSample contains information about the JVM at a specific point in time.
 *
 * @author Bart Cremers
 * @since 20-nov-2007
 */
public final class GCSample {

    private final long newGenMaxSize;
    private final long newGenMinSize;
    private final long newGenCurSize;
    private final long edenSize;
    private final long edenCapacity;
    private final long edenUsed;
    private final long edenGCEvents;
    private final long edenGCTime;
    private final long survivor0Size;
    private final long survivor0Capacity;
    private final long survivor0Used;
    private final long survivor1Size;
    private final long survivor1Capacity;
    private final long survivor1Used;
    private final long tenuredSize;
    private final long tenuredCapacity;
    private final long tenuredUsed;
    private final long tenuredGCEvents;
    private final long tenuredGCTime;
    private final long permSize;
    private final long permCapacity;
    private final long permUsed;
    private final long tenuringThreshold;
    private final long desiredSurvivorSize;
    private final long[] ageTableSizes;
    private final long classLoadTime;
    private final long classesLoaded;
    private final long classesUnloaded;
    private final long classBytesLoaded;
    private final long classBytesUnloaded;
    private final long totalCompileTime;
    private final long totalCompile;
    private final boolean finalizerInitialized;
    private final long finalizerTime;
    private final long finalizerCount;
    private final long finalizerQLength;
    private final long finalizerQMaxLength;
    private final long osElapsedTime;
    private final long lastModificationTime;
    private final String lastGCCause;
    private final String currentGCCause;

    private static long maxTenuringThreshold;
    private static long osFrequency;
    private static String javaCommand;
    private static String javaHome;
    private static String vmArgs;
    private static String vmFlags;
    private static String vmInfo;
    private static String vmName;
    private static String vmVersion;
    private static String vmVendor;
    private static String vmSpecName;
    private static String vmSpecVersion;
    private static String vmSpecVendor;
    private static String classPath;
    private static String bootClassPath;
    private static String libraryPath;
    private static String bootLibraryPath;
    private static String endorsedDirs;
    private static String extDirs;

    private static boolean initialized = false;

    private static synchronized void initStaticCounters(Model model) {
        if (!initialized) {
            maxTenuringThreshold = model.getMaxTenuringThreshold();
            osFrequency = model.getOsFrequency();
            javaCommand = model.getJavaCommand();
            javaHome = model.getJavaHome();
            vmArgs = model.getVmArgs();
            vmFlags = model.getVmFlags();
            vmInfo = model.getVmInfo();
            vmName = model.getVmName();
            vmVersion = model.getVmVersion();
            vmVendor = model.getVmVendor();
            vmSpecName = model.getVmSpecName();
            vmSpecVersion = model.getVmSpecVersion();
            vmSpecVendor = model.getVmSpecVendor();
            classPath = model.getClassPath();
            bootClassPath = model.getBootClassPath();
            libraryPath = model.getLibraryPath();
            bootLibraryPath = model.getBootLibraryPath();
            endorsedDirs = model.getEndorsedDirs();
            extDirs = model.getExtDirs();
            initialized = true;
        }
    }

    public GCSample(Model model) {
        initStaticCounters(model);

        newGenMaxSize = model.getNewGenMaxSize();
        newGenMinSize = model.getNewGenMinSize();
        newGenCurSize = model.getNewGenCurSize();
        edenUsed = model.getEdenUsed();
        survivor0Used = model.getSurvivor0Used();
        survivor1Used = model.getSurvivor1Used();
        tenuredUsed = model.getTenuredUsed();
        permUsed = model.getPermUsed();
        tenuringThreshold = model.getTenuringThreshold();
        edenSize = model.getEdenSize();
        survivor0Size = model.getSurvivor0Size();
        survivor1Size = model.getSurvivor1Size();
        tenuredSize = model.getTenuredSize();
        permSize = model.getPermSize();
        edenCapacity = model.getEdenCapacity();
        survivor0Capacity = model.getSurvivor0Capacity();
        survivor1Capacity = model.getSurvivor1Capacity();
        tenuredCapacity = model.getTenuredCapacity();
        permCapacity = model.getPermCapacity();
        edenGCEvents = model.getEdenGCEvents();
        edenGCTime = model.getEdenGCTime();
        tenuredGCEvents = model.getTenuredGCEvents();
        tenuredGCTime = model.getTenuredGCTime();
        desiredSurvivorSize = model.getDesiredSurvivorSize();
        ageTableSizes = model.getAgeTableSizes();
        lastGCCause = model.getLastGCCause();
        currentGCCause = model.getCurrentGCCause();
        classLoadTime = model.getClassLoadTime();
        classesLoaded = model.getClassesLoaded();
        classesUnloaded = model.getClassesUnloaded();
        classBytesLoaded = model.getClassBytesLoaded();
        classBytesUnloaded = model.getClassBytesUnloaded();
        totalCompileTime = model.getTotalCompileTime();
        totalCompile = model.getTotalCompile();

        model.initializeFinalizer();

        finalizerInitialized = model.isFinalizerInitialized();
        finalizerTime = model.getFinalizerTime();
        finalizerCount = model.getFinalizerCount();
        finalizerQLength = model.getFinalizerQLength();
        finalizerQMaxLength = model.getFinalizerQMaxLength();
        osElapsedTime = model.getOsElapsedTime();
        lastModificationTime = model.getLastModificationTime();
    }

    public static long getOsFrequency() {
        return osFrequency;
    }

    public static long getMaxTenuringThreshold() {
        return maxTenuringThreshold;
    }

    public static String getJavaCommand() {
        return javaCommand;
    }

    public static String getVmArgs() {
        return vmArgs;
    }

    public static String getVmFlags() {
        return vmFlags;
    }

    public static String getJavaHome() {
        return javaHome;
    }

    public static String getClassPath() {
        return classPath;
    }

    public static String getLibraryPath() {
        return libraryPath;
    }

    public static String getEndorsedDirs() {
        return endorsedDirs;
    }

    public static String getExtDirs() {
        return extDirs;
    }

    public static String getBootClassPath() {
        return bootClassPath;
    }

    public static String getBootLibraryPath() {
        return bootLibraryPath;
    }

    public static String getVmName() {
        return vmName;
    }

    public static String getVmInfo() {
        return vmInfo;
    }

    public static String getVmVendor() {
        return vmVendor;
    }

    public static String getVmVersion() {
        return vmVersion;
    }

    public static String getVmSpecName() {
        return vmSpecName;
    }

    public static String getVmSpecVendor() {
        return vmSpecVendor;
    }

    public static String vetVmSpecVersion() {
        return vmSpecVersion;
    }

    public double getAdjustedEdenSize() {
        if (edenCapacity + survivor0Capacity + survivor1Capacity == newGenMaxSize) {
            return (double) edenCapacity;
        } else {
            long diff = newGenMaxSize - newGenCurSize;
            return (double) (edenCapacity + diff);
        }
    }

    public double getEdenLiveRatio() {
        return (double) edenUsed / (double) edenSize;
    }

    public double getAdjustedEdenLiveRatio() {
        return (double) edenUsed / getAdjustedEdenSize();
    }

    public double getSurvivor0LiveRatio() {
        return (double) survivor0Used / (double) survivor0Size;
    }

    public double getSurvivor1LiveRatio() {
        return (double) survivor1Used / (double) survivor1Size;
    }

    public double getTenuredLiveRatio() {
        return (double) tenuredUsed / (double) tenuredSize;
    }

    public double getPermLiveRatio() {
        return (double) permUsed / (double) permSize;
    }

    public double getAdjustedEdenCommittedRatio() {
        return (double) edenCapacity / getAdjustedEdenSize();
    }

    public double getEdenCommittedRatio() {
        return (double) edenCapacity / (double) edenSize;
    }

    public double getSurvivor0CommittedRatio() {
        return (double) survivor0Capacity / (double) survivor0Size;
    }

    public double getSurvivor1CommittedRatio() {
        return (double) survivor1Capacity / (double) survivor1Size;
    }

    public double getTenuredCommittedRatio() {
        return (double) tenuredCapacity / (double) tenuredSize;
    }

    public double getPermCommittedRatio() {
        return (double) permCapacity / (double) permSize;
    }

    public long getTotalGCTime(GCSample gcsample) {
        return Math.abs(edenGCTime - gcsample.edenGCTime) + Math.abs(tenuredGCTime - gcsample.tenuredGCTime);
    }

    public boolean heapSizeChanged(GCSample gcsample) {
        return edenCapacity != gcsample.edenCapacity || survivor0Capacity != gcsample.survivor0Capacity
               || survivor1Capacity != gcsample.survivor1Capacity || tenuredCapacity != gcsample.tenuredCapacity
               || permCapacity != gcsample.permCapacity;
    }

    public long[] getAgeTableSizes() {
        return ageTableSizes;
    }

    public long getTenuringThreshold() {
        return tenuringThreshold;
    }

    public long getEdenGCTime() {
        return edenGCTime;
    }

    public long getTenuredGCTime() {
        return tenuredGCTime;
    }

    public boolean isFinalizerInitialized() {
        return finalizerInitialized;
    }

    public long getEdenSize() {
        return edenSize;
    }

    public long getEdenCapacity() {
        return edenCapacity;
    }

    public long getSurvivor0Size() {
        return survivor0Size;
    }

    public long getSurvivor0Capacity() {
        return survivor0Capacity;
    }

    public long getSurvivor1Size() {
        return survivor1Size;
    }

    public long getSurvivor1Capacity() {
        return survivor1Capacity;
    }

    public long getTenuredSize() {
        return tenuredSize;
    }

    public long getTenuredCapacity() {
        return tenuredCapacity;
    }

    public long getPermSize() {
        return permSize;
    }

    public long getPermCapacity() {
        return permCapacity;
    }

    public long getLastModificationTime() {
        return lastModificationTime;
    }

    public long getPermUsed() {
        return permUsed;
    }

    public long getTenuredUsed() {
        return tenuredUsed;
    }

    public long getEdenUsed() {
        return edenUsed;
    }

    public long getNewGenMaxSize() {
        return newGenMaxSize;
    }

    public long getNewGenCurSize() {
        return newGenCurSize;
    }

    public long getSurvivor0Used() {
        return survivor0Used;
    }

    public long getSurvivor1Used() {
        return survivor1Used;
    }

    public long getEdenGCEvents() {
        return edenGCEvents;
    }

    public long getTenuredGCEvents() {
        return tenuredGCEvents;
    }

    public String getLastGCCause() {
        return lastGCCause;
    }

    public long getClassesLoaded() {
        return classesLoaded;
    }

    public long getClassesUnloaded() {
        return classesUnloaded;
    }

    public long getClassLoadTime() {
        return classLoadTime;
    }

    public long getTotalCompileTime() {
        return totalCompileTime;
    }

    public long getTotalCompile() {
        return totalCompile;
    }

    public long getFinalizerTime() {
        return finalizerTime;
    }

    public long getFinalizerCount() {
        return finalizerCount;
    }

    public long getFinalizerQLength() {
        return finalizerQLength;
    }

    public long getFinalizerQMaxLength() {
        return finalizerQMaxLength;
    }

    public long getDesiredSurvivorSize() {
        return desiredSurvivorSize;
    }

    public long getOsElapsedTime() {
        return osElapsedTime;
    }

    public long getClassBytesLoaded() {
        return classBytesLoaded;
    }

    public long getClassBytesUnloaded() {
        return classBytesUnloaded;
    }

    public String getCurrentGCCause() {
        return currentGCCause;
    }

    public static boolean isInitialized() {
        return initialized;
    }

    public long getNewGenMinSize() {
        return newGenMinSize;
    }

    public static String getVmSpecVersion() {
        return vmSpecVersion;
    }

    public String getFinalizerQString() {
        return "Finalizer Time: " + getFinalizerCount() + " objects - " + Converter.longToTimeString(getFinalizerTime(),
                                                                                                     GCSample.getOsFrequency());
    }

    public String getFinalizerString() {
        return "Finalizer Time: " + getFinalizerCount() + " objects - " + Converter.longToTimeString(getFinalizerTime(),
                                                                                                     GCSample.getOsFrequency());
    }

    public String getCompileString() {
        return "Compile Time: " + getTotalCompile() + " compiles - " + Converter.longToTimeString(getTotalCompileTime(),
                                                                                                  GCSample.getOsFrequency());
    }

    public String getClassLoaderString() {
        return "Class Loader Time: " + getClassesLoaded() + " loaded, " + getClassesUnloaded() + " unloaded - "
               + Converter.longToTimeString(getClassLoadTime(), GCSample.getOsFrequency());
    }

    public String getGCString() {
        String s = "GC Time: " + (getEdenGCEvents() + getTenuredGCEvents()) + " collections, " + Converter
            .longToTimeString(getEdenGCTime() + getTenuredGCTime(), GCSample.getOsFrequency());

        if (getLastGCCause() != null && getLastGCCause().length() != 0) {
            s = s + " Last Cause: " + getLastGCCause();
        }

        return s;
    }
}