/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui;

import static net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration.*;
import net.sf.intelliplugin.visualgc.model.GCSample;
import net.sf.intelliplugin.visualgc.ui.graph.Level;
import net.sf.intelliplugin.visualgc.util.BorderUtil;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * A panel containing a histogram with information about the survivor spaces. It shows the amount of objects in each
 * age bucket.
 * 
 * @author Bart Cremers
 * @since 21-nov-2007
 */
class AgeHistogram extends JPanel implements ActionListener, ComponentListener, PropertyChangeListener {

    private static final int BUCKET_WIDTH = 20;
    private static final int BUCKET_HEIGHT = 40;
    private GCSample currentSample;
    private Level[] bucketLevel;
    private JPanel textPanel;
    private JPanel histogramPanel;
    private JLabel ttField;
    private JLabel mttField;
    private JLabel dssField;
    private JLabel cssField;

    public AgeHistogram(GCSample gcsample) {
        currentSample = gcsample;

        createTextPanel(gcsample);

        createHistogramPanel(gcsample);

        resetPanel();

        getConfiguration().addPropertyChangeListener(this);
    }

    private void createTextPanel(GCSample gcsample) {
        JLabel ttLabel = createLabel("Tenuring Threshold: ");
        ttField = createLabel(String.valueOf(gcsample.getTenuringThreshold()));
        ttLabel.setLabelFor(ttField);

        JLabel mttLabel = createLabel("Max Tenuring Threshold: ");
        mttField = createLabel(String.valueOf(GCSample.getMaxTenuringThreshold()));
        mttLabel.setLabelFor(mttField);

        JLabel dssLabel = createLabel("Desired Survivor Size: ");
        dssField = createLabel(String.valueOf(gcsample.getDesiredSurvivorSize()));
        dssLabel.setLabelFor(dssField);

        JLabel cssLabel = createLabel("Current Survivor Size: ");
        cssField = createLabel(String.valueOf(gcsample.getDesiredSurvivorSize()));
        cssLabel.setLabelFor(cssField);

        textPanel = new JPanel(new GridLayout(1, 0));
        textPanel.setBackground(getConfiguration().getBackgroundColor());
        textPanel.setBorder(BorderUtil.createTitledBorder("Parameters", getConfiguration().getSurvivorColor()));

        textPanel.add(ttLabel);
        textPanel.add(ttField);
        textPanel.add(mttLabel);
        textPanel.add(mttField);
        textPanel.add(dssLabel);
        textPanel.add(dssField);
        textPanel.add(cssLabel);
        textPanel.add(cssField);
    }

    private JLabel createLabel(String text) {
        JLabel ttLabel = new JLabel(text);
        ttLabel.setFont(getConfiguration().getBorderFont());
        ttLabel.setForeground(getConfiguration().getSurvivorColor());
        return ttLabel;
    }

    private void createHistogramPanel(GCSample gcsample) {
        histogramPanel = new JPanel();
        Color bgColor = getConfiguration().getBackgroundColor();
        histogramPanel.setBackground(bgColor);
        GridBagLayout gridbaglayout = new GridBagLayout();
        histogramPanel.setLayout(gridbaglayout);
        histogramPanel.setBorder(BorderUtil.createTitledBorder("Histogram", getConfiguration().getSurvivorColor()));
        Dimension dimension = new Dimension(BUCKET_WIDTH, BUCKET_HEIGHT);

        int bucketCount = gcsample.getAgeTableSizes().length;
        bucketLevel = new Level[bucketCount];

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = 0;
        gbc.weighty = 1.0D;
        gbc.weightx = 1.0D / (double) bucketCount;

        for (int i = 0; i < bucketCount; i++) {
            JPanel panel = new JPanel(new GridLayout(1, 1));
            panel.setBackground(bgColor);
            panel.setBorder(BorderUtil.createTitledBorder("" + i, getConfiguration().getSurvivorColor()));
            bucketLevel[i] = new Level(getConfiguration().getSurvivorColor());
            bucketLevel[i].setMaximumSize(dimension);
            bucketLevel[i].setMaximumSize(dimension);
            bucketLevel[i].setPreferredSize(dimension);
            panel.add(bucketLevel[i]);
            if (i == bucketCount - 1) {
                gbc.gridwidth = GridBagConstraints.REMAINDER;
            }
            histogramPanel.add(panel, gbc);
        }
    }

    void resetPanel() {
        removeAll();
        GridBagLayout layout = new GridBagLayout();
        setLayout(layout);
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = 1;
        gbc.gridwidth = 0;
        gbc.gridheight = -1;
        gbc.weighty = 0.1;
        gbc.weightx = 1.0;
        layout.setConstraints(textPanel, gbc);

        gbc.weighty = 0.9;
        layout.setConstraints(histogramPanel, gbc);

        add(textPanel);
        add(histogramPanel);
    }

    private void resetSpace() {
        resetPanel();
    }

    void refreshPanels() {
        repaint();
    }

    void updateLevel(GCSample gcsample) {
        int i = gcsample.getAgeTableSizes().length;
        for (int j = 0; j < i; j++) {
            double d = (double) gcsample.getAgeTableSizes()[j] / (double) gcsample.getSurvivor0Size();
            bucketLevel[j].updateLevel(d);
        }

        ttField.setText(String.valueOf(gcsample.getTenuringThreshold()));
        mttField.setText(String.valueOf(GCSample.getMaxTenuringThreshold()));
        dssField.setText(String.valueOf(gcsample.getDesiredSurvivorSize()));
        cssField.setText(String.valueOf(gcsample.getSurvivor0Capacity()));
    }

    public void update(GCSample gcsample) {
        if (gcsample.heapSizeChanged(currentSample)) {
            resetSpace();
        }
        updateLevel(gcsample);
        refreshPanels();
        currentSample = gcsample;
    }

    public void componentHidden(ComponentEvent componentevent) {
    }

    public void componentMoved(ComponentEvent componentevent) {
    }

    public void componentShown(ComponentEvent componentevent) {
    }

    public void componentResized(ComponentEvent componentevent) {
        resetSpace();
        validate();
    }

    public void actionPerformed(ActionEvent actionevent) {
    }

    public void propertyChange(PropertyChangeEvent evt) {
        String pName = evt.getPropertyName();
        if (BACKGROUND_COLOR.equals(pName)) {
            Color bgColor = getConfiguration().getBackgroundColor();

            textPanel.setBackground(bgColor);
            histogramPanel.setBackground(bgColor);

            for (Component comp : histogramPanel.getComponents()) {
                comp.setBackground(bgColor);
            }
        } else if (SURVIVOR_COLOR.equals(pName)) {
            Color color = getConfiguration().getSurvivorColor();
            BorderUtil.changeTitledBorderColor(textPanel, color);
            for (Component comp : textPanel.getComponents()) {
                comp.setForeground(color);
            }
            BorderUtil.changeTitledBorderColor(histogramPanel, color);
            for (Component comp : histogramPanel.getComponents()) {
                JPanel panel = (JPanel) comp;
                ((Level) panel.getComponent(0)).setColor(color);
                BorderUtil.changeTitledBorderColor(panel, color);
            }
        } else if (BORDER_COLOR.equalsIgnoreCase(pName)) {
            Color color = getConfiguration().getSurvivorColor();
            BorderUtil.changeTitledBorderColor(textPanel, color);
            BorderUtil.changeTitledBorderColor(histogramPanel, color);
            for (Component comp : histogramPanel.getComponents()) {
                JPanel panel = (JPanel) comp;
                BorderUtil.changeTitledBorderColor(panel, color);
            }
        } else if (PRIMARY_GRID_COLOR.equalsIgnoreCase(pName)
                   || SECUNDARY_GRID_COLOR.equalsIgnoreCase(pName)) {
            Color primaryColor = getConfiguration().getPrimaryGridColor();
            Color secundaryColor = getConfiguration().getSecundaryGridColor();
            for (Component comp : histogramPanel.getComponents()) {
                JPanel panel = (JPanel) comp;
                ((Level) panel.getComponent(0)).setGridColors(primaryColor, secundaryColor);
            }
        }
    }
}