/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui;

import static net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration.getConfiguration;
import net.sf.intelliplugin.visualgc.ui.graph.FIFORangedList;
import net.sf.intelliplugin.visualgc.ui.graph.Line;
import net.sf.intelliplugin.visualgc.util.BorderUtil;
import net.sf.intelliplugin.visualgc.util.Converter;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * A panel containing information about a single heap space over time. This graph also contains information about the
 * occured garbage collections.
 *  
 * @author Bart Cremers
 * @since 20-nov-2007
 */
class GCSpacePanel extends JPanel {

    private static final int DATASET_SIZE = 1000;
    private final Line line;
    private final String borderString;
    private final FIFORangedList dataset;
    private boolean reservedMode;

    public GCSpacePanel(String s, long reservedSize, long currentSize, Color color) {
        this(s, reservedSize, currentSize, color, false);
    }

    public GCSpacePanel(String s, long reservedSize, long currentSize, Color color, boolean reservedMode) {
        super(new BorderLayout());

        this.reservedMode = reservedMode;
        borderString = s + " (" + Converter.longToKMGString(reservedSize) + ", ";

        if (reservedMode) {
            dataset = new FIFORangedList(DATASET_SIZE, 0.0D, reservedSize);
        } else {
            dataset = new FIFORangedList(DATASET_SIZE, 0.0D, currentSize);
        }

        line = new Line(dataset, color);

        if (reservedMode) {
            line.updateGrayLevel(1.0D - (double) currentSize / (double) reservedSize);
        }

        String borderTitle = borderString + Converter.longToKMGString(currentSize) + "): " + " Used = ";

        setBorder(BorderUtil.createTitledBorder(borderTitle, color));
        setForeground(color);
        setBackground(getConfiguration().getBackgroundColor());
        setPreferredSize(new Dimension(600, 100));
        add(line, BorderLayout.CENTER);

        addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent mouseevent) {
                maybeShowPopup(mouseevent);
            }

            public void mouseReleased(MouseEvent mouseevent) {
                maybeShowPopup(mouseevent);
            }

            private void maybeShowPopup(MouseEvent mouseevent) {
                if (mouseevent.isPopupTrigger()) {
                    JPopupMenu menu = new JPopupMenu();
                    JCheckBoxMenuItem menuItem = new JCheckBoxMenuItem("Show Reserved Space", isShowReservedSpace());
                    menuItem.addItemListener(new ItemListener() {
                        public void itemStateChanged(ItemEvent itemevent) {
                            setShowReservedSpace(itemevent.getStateChange() == 1);
                        }
                    });
                    menu.add(menuItem);
                    menu.show(GCSpacePanel.this, mouseevent.getX(), mouseevent.getY());
                }
            }
        });
    }

    public Line getLine() {
        return line;
    }

    public void setShowReservedSpace(boolean flag) {
        reservedMode = flag;
    }

    public boolean isShowReservedSpace() {
        return reservedMode;
    }

    public void updateGraph(long reservedSize, long currentSize, long currentLevel) {
        dataset.add((double) currentLevel);
        if (isShowReservedSpace()) {
            dataset.setMaxValue(reservedSize);
            line.updateGrayLevel(1.0D - (double) currentSize / (double) reservedSize);
        } else {
            dataset.setMaxValue(currentSize);
            line.updateGrayLevel(0.0D);
        }
    }

    public void updateGraph(long reservedSize, long currentSize, long currentLevel, long sizeDelta) {
        dataset.add((double) currentLevel);
        if (isShowReservedSpace()) {
            long totalSize = reservedSize + sizeDelta;
            dataset.setMaxValue(totalSize);
            line.updateGrayLevel(1.0D - (double) currentSize / (double) totalSize);
        } else {
            dataset.setMaxValue(currentSize);
            line.updateGrayLevel(0.0D);
        }
    }

    public void updateTextComponents(long capacity, long used) {

        TitledBorder titledborder = (TitledBorder) getBorder();
        Color color = getForeground();

        if (!titledborder.getTitleColor().equals(color)) {
            Border border = BorderFactory.createEtchedBorder(color, getConfiguration().getBorderColor());
            titledborder.setTitleColor(color);
            titledborder.setBorder(border);
        }

        String s = borderString + Converter.longToKMGString(capacity) + "): " + Converter.longToKMGString(used);

        titledborder.setTitle(s);
    }

    // gcSample.getTenuredCapacity(), gcSample.getTenuredUsed(),
    //       gcSample.getTenuredGCEvents(), gcSample.getTenuredGCTime(),
    //       GCSample.getOsFrequency()
    public void updateTextComponents(long capacity, long used, long gcEvents, long gcTime, long frequency) {

        TitledBorder titledborder = (TitledBorder) getBorder();
        Color color = getForeground();

        if (!titledborder.getTitleColor().equals(color)) {
            Border border = BorderFactory.createEtchedBorder(color, getConfiguration().getBorderColor());
            titledborder.setTitleColor(color);
            titledborder.setBorder(border);
        }

        String s = borderString + Converter.longToKMGString(capacity) + "): " + Converter.longToKMGString(used) + ", " + gcEvents
                   + " collections, " + Converter.longToTimeString(gcTime, frequency);
        titledborder.setTitle(s);
    }
}