/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

package net.sf.intelliplugin.visualgc.ui;

import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.options.Configurable;
import com.intellij.openapi.options.ConfigurationException;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.project.ProjectManager;
import com.intellij.openapi.project.ProjectManagerAdapter;
import com.intellij.openapi.ui.ComponentWithBrowseButton;
import com.intellij.openapi.util.IconLoader;
import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.NamedJDOMExternalizable;
import com.intellij.openapi.util.WriteExternalException;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.peer.PeerFactory;
import com.intellij.ui.TextFieldWithStoredHistory;
import com.intellij.ui.content.Content;
import net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration;
import net.sf.intelliplugin.visualgc.model.GCSample;
import net.sf.intelliplugin.visualgc.model.Model;
import net.sf.intelliplugin.visualgc.model.MonitorFactory;
import net.sf.intelliplugin.visualgc.util.JDOMExternalizer;
import net.sf.intelliplugin.visualgc.util.DialogHelper;
import org.jdom.Element;
import org.jetbrains.annotations.Nls;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import sun.jvmstat.monitor.MonitorException;
import sun.jvmstat.monitor.MonitoredHost;
import sun.jvmstat.monitor.MonitoredVm;
import sun.jvmstat.monitor.VmIdentifier;
import sun.jvmstat.monitor.event.HostEvent;
import sun.jvmstat.monitor.event.HostListener;
import sun.jvmstat.monitor.event.VmStatusChangeEvent;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URISyntaxException;
import java.util.HashMap;

/**
 * The ApplicationComponent. 
 */
@SuppressWarnings ({ "deprecation" })
public class IntelliHeapComponent implements ApplicationComponent, ActionListener, Configurable, NamedJDOMExternalizable {

    private static final String DEFAULT_POLL_INTERVAL = "1000";

    private static final String CONFIGURATION_NAME = "intelli.heap"; 
    public static final String COMPONENT_NAME = "IntelliHeap";

    @SuppressWarnings ({ "MismatchedQueryAndUpdateOfCollection" })
    private static final HashMap<Project, ToolWindow> toolWindows = new HashMap<Project, ToolWindow>();

    private volatile boolean connected = false;

    private JPanel contentPane;

    private TextFieldWithStoredHistory tfIdentifier;
    private JTextField tfPollInterval;
    private JButton btConnect;

    private final JLabel lblDisconnected = new JLabel("Not connected", JLabel.CENTER);
    private VisualGCPanel visualGCPanel;

    private Timer timer;
    private Model monitoredvmmodel;
    private MonitoredVm monitoredvm;
    private GCSample currentSample;
    private ComponentWithBrowseButton<TextFieldWithStoredHistory> browseComponent;
    private java.util.List<String> restoredHistory;

    public IntelliHeapComponent() {
        lblDisconnected.setPreferredSize(new Dimension(1024, 768));
        lblDisconnected.setBorder(BorderFactory.createLoweredBevelBorder());
        lblDisconnected.setFont(lblDisconnected.getFont().deriveFont(22.0f));
    }

    public void initComponent() {
        ProjectManager.getInstance().addProjectManagerListener(new ProjectManagerAdapter() {
            public void projectOpened(Project project) {
                ToolWindowManager toolWindowManager = ToolWindowManager.getInstance(project);
                ToolWindow toolWindow = toolWindowManager.registerToolWindow(COMPONENT_NAME, true,
                                                                             ToolWindowAnchor.BOTTOM);
                toolWindow.setIcon(IconLoader.getIcon("/icon/ih-small.png", IntelliHeapComponent.class));
                toolWindow.setAutoHide(true);

                JPanel contentPane = getContentPane();
                PeerFactory peerFactory = PeerFactory.getInstance();
                Content content = peerFactory.getContentFactory().createContent(contentPane, "", false);
                toolWindow.getContentManager().addContent(content);
                toolWindows.put(project, toolWindow);
            }

            public void projectClosed(Project project) {
                disconnect();
                toolWindows.remove(project);
                ToolWindowManager mgr = ToolWindowManager.getInstance(project);
                mgr.unregisterToolWindow(COMPONENT_NAME);
            }
        });
    }

    public void disposeComponent() {
    }

    public synchronized JPanel getContentPane() {
        if (contentPane == null) {
            final JPanel pane = new JPanel(new BorderLayout());
            JPanel topPane = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 5));

            tfIdentifier = new TextFieldWithStoredHistory(COMPONENT_NAME, true);
            if (restoredHistory != null) {
                tfIdentifier.setHistory(restoredHistory);
                restoredHistory = null;
            }
            // Hack to get button size right
            tfIdentifier.setBorder(((CompoundBorder) tfIdentifier.getBorder()).getInsideBorder());

            JLabel lblIdentifier = new JLabel("VM Identifier:");
            lblIdentifier.setLabelFor(tfIdentifier);
            lblIdentifier.setDisplayedMnemonic('v');

            tfPollInterval = new JTextField(DEFAULT_POLL_INTERVAL, 6);

            JLabel lblInterval = new JLabel("Poll Interval");
            lblIdentifier.setLabelFor(tfPollInterval);
            lblInterval.setDisplayedMnemonic('p');

            btConnect = new JButton("Connect");

            topPane.add(lblIdentifier);
            browseComponent = new ComponentWithBrowseButton<TextFieldWithStoredHistory>(tfIdentifier, this);

            topPane.add(browseComponent);
            topPane.add(Box.createHorizontalStrut(20));
            topPane.add(lblInterval);
            topPane.add(tfPollInterval);
            topPane.add(Box.createHorizontalStrut(20));
            topPane.add(btConnect);

            btConnect.addActionListener(this);

            pane.add(topPane, BorderLayout.NORTH);
            pane.add(lblDisconnected, BorderLayout.CENTER);
            contentPane = pane;
        }
        return contentPane;
    }

    private void connect(String text, int interval) throws URISyntaxException, MonitorException {
        VmIdentifier vmidentifier = new VmIdentifier(text);
        MonitoredHost monitoredhost = MonitoredHost.getMonitoredHost(vmidentifier);

        monitoredvm = monitoredhost.getMonitoredVm(vmidentifier, interval);
        monitoredvmmodel = MonitorFactory.createModel(monitoredvm);

        currentSample = new GCSample(monitoredvmmodel);

        monitoredhost.addHostListener(new HostListener() {
            public void vmStatusChanged(VmStatusChangeEvent vmStatusChangeEvent) {
            }

            public void disconnected(HostEvent hostEvent) {
                try {
                    hostEvent.getMonitoredHost().detach(monitoredvm);
                } catch (MonitorException e) {
                    // Ignore
                }
            }
        });

        visualGCPanel = new VisualGCPanel(currentSample);
        contentPane.remove(lblDisconnected);
        contentPane.add(visualGCPanel, BorderLayout.CENTER);
    }

    @NotNull
    public String getComponentName() {
        return COMPONENT_NAME;
    }

    public void actionPerformed(ActionEvent e) {
        if (btConnect.equals(e.getSource())) {
            if (connected) {
                disconnect();

                contentPane.remove(visualGCPanel);
                contentPane.add(lblDisconnected, BorderLayout.CENTER);

                btConnect.setText("Connect");
                browseComponent.setEnabled(true);
                tfPollInterval.setEnabled(true);
                tfIdentifier.setEnabled(true);
                tfIdentifier.getTextEditor().setEnabled(true);
                btConnect.setEnabled(true);

                tfIdentifier.getTextEditor().requestFocusInWindow();
            } else {
                try {
                    browseComponent.setEnabled(false);
                    tfIdentifier.setEnabled(false);
                    tfIdentifier.getTextEditor().setEnabled(false);
                    tfPollInterval.setEnabled(false);
                    btConnect.setEnabled(false);

                    int interval = Integer.parseInt(tfPollInterval.getText().trim());
                    connect(tfIdentifier.getText(), interval);

                    timer = new Timer(interval, this);
                    timer.start();

                    btConnect.setText("Disconnect");
                    btConnect.setEnabled(true);
                    connected = true;
                    tfIdentifier.addCurrentTextToHistory();
                } catch (Exception ex) {
                    JOptionPane.showMessageDialog(contentPane, ex.getMessage(), "Error Connecting to VM",
                                                  JOptionPane.ERROR_MESSAGE);
                    browseComponent.setEnabled(true);
                    tfIdentifier.setEnabled(true);
                    tfIdentifier.getTextEditor().setEnabled(true);
                    tfPollInterval.setEnabled(true);
                    btConnect.setEnabled(true);
                }
            }
            lblDisconnected.repaint();
        } else if (timer != null && timer.equals(e.getSource())) {
            currentSample = new GCSample(monitoredvmmodel);
            visualGCPanel.update(currentSample);
        } else if (browseComponent.getButton().equals(e.getSource())) {
            final VMIdentifierPanel vmIdentifierPanel = new VMIdentifierPanel();

            JPanel buttons = new JPanel(new FlowLayout(FlowLayout.RIGHT));
            buttons.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

            JPanel buttonGrid = new JPanel(new GridLayout(1, 0, 5, 5));

            JButton okButton = new JButton("OK");
            JButton cancelButton = new JButton("Cancel");
            buttonGrid.add(okButton);
            buttonGrid.add(cancelButton);

            buttons.add(buttonGrid);

            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    tfIdentifier.setText(vmIdentifierPanel.getVMIdentifier());
                    SwingUtilities.getWindowAncestor((Component) e.getSource()).dispose();
                }
            });

            cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    SwingUtilities.getWindowAncestor((Component) e.getSource()).dispose();
                }
            });

            DialogHelper.showInDialog("VM Identifier", vmIdentifierPanel, buttons);
        }
    }

    private void disconnect() {
        if (timer != null) {
            timer.stop();
        }
        timer = null;
        connected = false;
    }

    public MonitoredVm getMonitoredvm() {
        return monitoredvm;
    }

    public GCSample getCurrentSample() {
        return currentSample;
    }

    @Nls
    public String getDisplayName() {
        return COMPONENT_NAME;
    }

    @Nullable
    public Icon getIcon() {
        return IconLoader.getIcon("/icon/ih.png", IntelliHeapComponent.class);
    }

    @Nullable
    @NonNls
    public String getHelpTopic() {
        return null;
    }

    public JComponent createComponent() {
        return IntelliHeapConfiguration.getConfiguration().getConfigurationPanel();
    }

    public boolean isModified() {
        return IntelliHeapConfiguration.getConfiguration().isModified();
    }

    public void apply() throws ConfigurationException {
        IntelliHeapConfiguration.getConfiguration().apply();
    }

    public void reset() {
        IntelliHeapConfiguration.getConfiguration().reset();
    }

    public void disposeUIResources() {
    }

    public void readExternal(Element root) throws InvalidDataException {
        IntelliHeapConfiguration.getConfiguration().readExternal(root);
        restoredHistory = JDOMExternalizer.readList(root.getChild("history"));
    }

    @SuppressWarnings ({ "unchecked" })
    public void writeExternal(Element root) throws WriteExternalException {
        IntelliHeapConfiguration.getConfiguration().writeExternal(root);

        Element history = new Element("history");

        JDOMExternalizer.writeList(history, tfIdentifier.getHistory());

        root.addContent(history);
    }

    @NonNls
    public String getExternalFileName() {
        return CONFIGURATION_NAME;
    }
}