/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui;

import static net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration.*;
import net.sf.intelliplugin.visualgc.model.GCSample;
import net.sf.intelliplugin.visualgc.ui.graph.FIFORangedList;
import net.sf.intelliplugin.visualgc.ui.graph.Line;
import net.sf.intelliplugin.visualgc.util.BorderUtil;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * A graph containing the information about the Heap over time.
 *
 * @author Bart Cremers
 * @since 20-nov-2007
 */
class RealTime extends JPanel implements ComponentListener, PropertyChangeListener {

    private final JPanel heapPanel;
    private final JPanel timePanel;
    private final JPanel gcPanel;
    private final JPanel classPanel;
    private final JPanel compilePanel;
    private final JPanel finalizerPanel;
    private final JPanel finalizerQPanel;

    private final GCSpacePanel permPanel;
    private final GCSpacePanel oldPanel;
    private final GCSpacePanel edenPanel;
    private final GCSpacePanel s0Panel;
    private final GCSpacePanel s1Panel;

    private final FIFORangedList gcActiveDataSet;
    private final FIFORangedList finalizerActiveDataSet;
    private final FIFORangedList finalizerQLengthDataSet;
    private final FIFORangedList compilerActiveDataSet;
    private final FIFORangedList classLoaderActiveDataSet;

    private GCSample currentSample;

    private boolean inGC = false;
    private boolean inEdGC = false;
    private boolean inTnGC = false;

    private long edenGCStart;
    private long tenuredGCStart;
    private long maxFinalizerQLength;

    public RealTime(GCSample gcSample) {
        currentSample = gcSample;

        edenGCStart = gcSample.getEdenGCTime();
        tenuredGCStart = gcSample.getTenuredGCTime();

        heapPanel = new JPanel();
        heapPanel.setBackground(getConfiguration().getBackgroundColor());

        gcActiveDataSet = new FIFORangedList(1000, 0.0D, 1.0D);
        classLoaderActiveDataSet = new FIFORangedList(1000, 0.0D, 1.0D);
        compilerActiveDataSet = new FIFORangedList(1000, 0.0D, 1.0D);

        gcPanel = createLinePanel(gcActiveDataSet, getConfiguration().getGcColor());
        classPanel = createLinePanel(classLoaderActiveDataSet, getConfiguration().getLoaderColor());
        compilePanel = createLinePanel(compilerActiveDataSet, getConfiguration().getHotspotColor());

        timePanel = new JPanel();
        timePanel.setBackground(Color.BLACK);
        timePanel.setLayout(new GridLayout(5, 1));

        finalizerActiveDataSet = new FIFORangedList(1000, 0.0D, 1.0D);
        finalizerQLengthDataSet = new FIFORangedList(1000);

        finalizerPanel = createLinePanel(finalizerActiveDataSet, getConfiguration().getFinalizerColor());
        finalizerQPanel = createLinePanel(finalizerQLengthDataSet, getConfiguration().getFinalizerColor());

        if (gcSample.isFinalizerInitialized()) {
            timePanel.setLayout(new GridLayout(5, 1));
            timePanel.add(finalizerQPanel);
            timePanel.add(finalizerPanel);
        } else {
            timePanel.setLayout(new GridLayout(3, 1));
        }

        timePanel.add(compilePanel);
        timePanel.add(classPanel);
        timePanel.add(gcPanel);

        edenPanel = new GCSpacePanel("Eden Space", gcSample.getEdenSize(), gcSample.getEdenCapacity(),
                                     getConfiguration().getEdenColor());

        s0Panel = new GCSpacePanel("Survivor 0", gcSample.getSurvivor0Size(), gcSample.getSurvivor0Capacity(),
                                   getConfiguration().getSurvivorColor());
        s1Panel = new GCSpacePanel("Survivor 1", gcSample.getSurvivor1Size(), gcSample.getSurvivor1Capacity(),
                                   getConfiguration().getSurvivorColor());

        oldPanel = new GCSpacePanel("Old Gen", gcSample.getTenuredSize(), gcSample.getTenuredCapacity(),
                                    getConfiguration().getTenuredColor());
        permPanel = new GCSpacePanel("Perm Gen", gcSample.getPermSize(), gcSample.getPermCapacity(),
                                     getConfiguration().getPermColor());

        addComponentListener(this);
        update(gcSample);
        resetPanel(gcSample);

        getConfiguration().addPropertyChangeListener(this);
    }

    private JPanel createLinePanel(FIFORangedList dataSet, Color color) {
        JPanel panel = new JPanel(new GridLayout(1, 1));
        panel.setBackground(getConfiguration().getBackgroundColor());

        Line line = new Line(dataSet, color);

        panel.setBorder(BorderUtil.createTitledBorder("", color));
        panel.add(line);

        return panel;
    }

    public void componentHidden(ComponentEvent evt) {
    }

    public void componentMoved(ComponentEvent evt) {
    }

    public void componentShown(ComponentEvent evt) {
    }

    public void componentResized(ComponentEvent evt) {
        resetPanel(currentSample);
        validate();
    }

    public void update(GCSample gcsample) {
        if (gcsample.getLastModificationTime() != currentSample.getLastModificationTime()) {
            resetPanel(gcsample);
            validate();
        }

        updateGraph(gcsample);
        updateTextComponents(gcsample);
        refreshPanels();

        currentSample = gcsample;
    }

    private void refreshPanels() {
        heapPanel.repaint();
        timePanel.repaint();
    }

    private void updateGraph(GCSample gcSample) {
        permPanel.updateGraph(gcSample.getPermSize(), gcSample.getPermCapacity(), gcSample.getPermUsed());
        oldPanel.updateGraph(gcSample.getTenuredSize(), gcSample.getTenuredCapacity(), gcSample.getTenuredUsed());
        edenPanel.updateGraph(gcSample.getEdenSize(), gcSample.getEdenCapacity(), gcSample.getEdenUsed(),
                              gcSample.getNewGenMaxSize() - gcSample.getNewGenCurSize());
        s0Panel.updateGraph(gcSample.getSurvivor0Size(), gcSample.getSurvivor0Capacity(), gcSample.getSurvivor0Used());
        s1Panel.updateGraph(gcSample.getSurvivor1Size(), gcSample.getSurvivor1Capacity(), gcSample.getSurvivor1Used());

        if (!inGC) {
            inEdGC = gcSample.getEdenGCEvents() != currentSample.getEdenGCEvents();
            inTnGC = gcSample.getTenuredGCEvents() != currentSample.getTenuredGCEvents();

            if (inEdGC || inTnGC) {
                inGC = true;

                if (inEdGC) {
                    edenGCStart = currentSample.getEdenGCTime();
                }

                if (inTnGC) {
                    tenuredGCStart = currentSample.getTenuredGCTime();
                }
            }
        }

        if (inGC) {

            if (inEdGC && gcSample.getEdenGCTime() != edenGCStart) {
                inEdGC = false;
            }

            if (inTnGC && gcSample.getTenuredGCTime() != tenuredGCStart) {
                inTnGC = false;
            }

            inGC = inEdGC || inTnGC;
        }

        gcActiveDataSet.add(1.0);

        finalizerQLengthDataSet.add((double) gcSample.getFinalizerQLength());

        int i = gcSample.getFinalizerTime() - currentSample.getFinalizerTime() != 0L ? 1 : 0;
        finalizerActiveDataSet.add((double) i);

        int j = gcSample.getClassLoadTime() - currentSample.getClassLoadTime() != 0L ? 1 : 0;
        classLoaderActiveDataSet.add((double) j);

        int k = gcSample.getTotalCompileTime() - currentSample.getTotalCompileTime() != 0L ? 1 : 0;
        compilerActiveDataSet.add((double) k);
    }

    private void updateTextComponents(GCSample gcSample) {

        maxFinalizerQLength = Math.max(maxFinalizerQLength, gcSample.getFinalizerQLength());

        setBorderTitle(finalizerQPanel.getBorder(), gcSample.getFinalizerQString());
        setBorderTitle(finalizerPanel.getBorder(), gcSample.getFinalizerString());
        setBorderTitle(compilePanel.getBorder(), gcSample.getCompileString());
        setBorderTitle(classPanel.getBorder(), gcSample.getClassLoaderString());
        setBorderTitle(gcPanel.getBorder(), gcSample.getGCString());

        permPanel.updateTextComponents(gcSample.getPermCapacity(), gcSample.getPermUsed());
        oldPanel.updateTextComponents(gcSample.getTenuredCapacity(), gcSample.getTenuredUsed(),
                                      gcSample.getTenuredGCEvents(), gcSample.getTenuredGCTime(),
                                      GCSample.getOsFrequency());
        edenPanel.updateTextComponents(gcSample.getEdenCapacity(), gcSample.getEdenUsed(), gcSample.getEdenGCEvents(),
                                       gcSample.getEdenGCTime(), GCSample.getOsFrequency());
        s0Panel.updateTextComponents(gcSample.getSurvivor0Capacity(), gcSample.getSurvivor0Used());
    }

    private void setBorderTitle(Border border, String title) {
        if (border instanceof TitledBorder) {
            ((TitledBorder) border).setTitle(title);
        }
    }

    void resetPanel(GCSample gcsample) {
        removeAll();
        heapPanel.removeAll();
        timePanel.removeAll();

        GridBagLayout layout = new GridBagLayout();
        heapPanel.setLayout(layout);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = 1;
        gbc.gridwidth = 0;
        gbc.weightx = 1.0D;

        gbc.weighty = 0.2;
        layout.setConstraints(permPanel, gbc);
        gbc.weighty = 0.3;
        layout.setConstraints(oldPanel, gbc);
        layout.setConstraints(edenPanel, gbc);
        gbc.weighty = 0.1;
        layout.setConstraints(s0Panel, gbc);
        layout.setConstraints(s1Panel, gbc);

        heapPanel.add(edenPanel);
        heapPanel.add(s0Panel);
        heapPanel.add(s1Panel);
        heapPanel.add(oldPanel);
        heapPanel.add(permPanel);

        if (gcsample.isFinalizerInitialized()) {
            timePanel.setLayout(new GridLayout(5, 1));

            gbc.weighty = 0.2;

            layout.setConstraints(finalizerQPanel, gbc);
            layout.setConstraints(finalizerPanel, gbc);
            layout.setConstraints(compilePanel, gbc);
            layout.setConstraints(classPanel, gbc);
            layout.setConstraints(gcPanel, gbc);

            timePanel.add(finalizerQPanel);
            timePanel.add(finalizerPanel);
        } else {
            timePanel.setLayout(new GridLayout(3, 1));
        }

        timePanel.add(compilePanel);
        timePanel.add(classPanel);
        timePanel.add(gcPanel);

        GridBagLayout outerLayout = new GridBagLayout();
        GridBagConstraints outerGbc = new GridBagConstraints();
        setLayout(outerLayout);
        outerGbc.fill = 1;
        outerGbc.gridwidth = 0;
        outerGbc.weighty = 0.2D;
        outerGbc.weightx = 1.0D;
        outerLayout.setConstraints(timePanel, outerGbc);
        outerGbc.gridheight = 0;
        outerGbc.weighty = 0.8D;
        outerGbc.weightx = 1.0D;
        outerLayout.setConstraints(heapPanel, outerGbc);
        add(timePanel);
        add(heapPanel);
    }

    public void propertyChange(PropertyChangeEvent evt) {
        String pName = evt.getPropertyName();

        if (BACKGROUND_COLOR.equals(pName)) {
            Color bgColor = getConfiguration().getBackgroundColor();
            heapPanel.setBackground(bgColor);
            timePanel.setBackground(bgColor);
            gcPanel.setBackground(bgColor);
            classPanel.setBackground(bgColor);
            compilePanel.setBackground(bgColor);
            finalizerPanel.setBackground(bgColor);
            finalizerQPanel.setBackground(bgColor);

            permPanel.setBackground(bgColor);
            oldPanel.setBackground(bgColor);
            edenPanel.setBackground(bgColor);
            s0Panel.setBackground(bgColor);
            s1Panel.setBackground(bgColor);
        } else if (GC_COLOR.equals(pName)) {
            changeLinePanelColor(gcPanel, getConfiguration().getGcColor());
        } else if (LOADER_COLOR.equals(pName)) {
            changeLinePanelColor(classPanel, getConfiguration().getLoaderColor());
        } else if (FINALIZER_COLOR.equals(pName)) {
            changeLinePanelColor(finalizerPanel, getConfiguration().getFinalizerColor());
            changeLinePanelColor(finalizerQPanel, getConfiguration().getFinalizerColor());
        } else if (HOTSPOT_COLOR.equals(pName)) {
            changeLinePanelColor(compilePanel, getConfiguration().getHotspotColor());
        } else if (PERM_COLOR.equals(pName)) {
            changeGCSpacePanelColor(permPanel, getConfiguration().getPermColor());
        } else if (TENURED_COLOR.equals(pName)) {
            changeGCSpacePanelColor(oldPanel, getConfiguration().getTenuredColor());
        } else if (SURVIVOR_COLOR.equals(pName)) {
            changeGCSpacePanelColor(s0Panel, getConfiguration().getSurvivorColor());
            changeGCSpacePanelColor(s1Panel, getConfiguration().getSurvivorColor());
        } else if (EDEN_COLOR.equals(pName)) {
            changeGCSpacePanelColor(edenPanel, getConfiguration().getEdenColor());
        } else if (BORDER_COLOR.equals(pName)) {
            BorderUtil.changeTitledBorderColor(gcPanel, getConfiguration().getGcColor());
            BorderUtil.changeTitledBorderColor(classPanel, getConfiguration().getLoaderColor());
            BorderUtil.changeTitledBorderColor(finalizerPanel, getConfiguration().getFinalizerColor());
            BorderUtil.changeTitledBorderColor(finalizerQPanel, getConfiguration().getFinalizerColor());
            BorderUtil.changeTitledBorderColor(compilePanel, getConfiguration().getHotspotColor());
            BorderUtil.changeTitledBorderColor(permPanel, getConfiguration().getPermColor());
            BorderUtil.changeTitledBorderColor(oldPanel, getConfiguration().getTenuredColor());
            BorderUtil.changeTitledBorderColor(s0Panel, getConfiguration().getSurvivorColor());
            BorderUtil.changeTitledBorderColor(s1Panel, getConfiguration().getSurvivorColor());
            BorderUtil.changeTitledBorderColor(edenPanel, getConfiguration().getEdenColor());
        } else if (PRIMARY_GRID_COLOR.equalsIgnoreCase(pName)
                   || SECUNDARY_GRID_COLOR.equalsIgnoreCase(pName)) {
            Color primaryColor = getConfiguration().getPrimaryGridColor();
            Color secundaryColor = getConfiguration().getSecundaryGridColor();

            edenPanel.getLine().setGridPrimaryColor(primaryColor);
            edenPanel.getLine().setGridSecundaryColor(secundaryColor);
            s1Panel.getLine().setGridPrimaryColor(primaryColor);
            s1Panel.getLine().setGridSecundaryColor(secundaryColor);
            s0Panel.getLine().setGridPrimaryColor(primaryColor);
            s0Panel.getLine().setGridSecundaryColor(secundaryColor);
            oldPanel.getLine().setGridPrimaryColor(primaryColor);
            oldPanel.getLine().setGridSecundaryColor(secundaryColor);
            permPanel.getLine().setGridPrimaryColor(primaryColor);
            permPanel.getLine().setGridSecundaryColor(secundaryColor);

            ((Line) gcPanel.getComponent(0)).setGridPrimaryColor(primaryColor);
            ((Line) gcPanel.getComponent(0)).setGridSecundaryColor(secundaryColor);
            ((Line) classPanel.getComponent(0)).setGridPrimaryColor(primaryColor);
            ((Line) classPanel.getComponent(0)).setGridSecundaryColor(secundaryColor);
            ((Line) finalizerPanel.getComponent(0)).setGridPrimaryColor(primaryColor);
            ((Line) finalizerPanel.getComponent(0)).setGridSecundaryColor(secundaryColor);
            ((Line) finalizerQPanel.getComponent(0)).setGridPrimaryColor(primaryColor);
            ((Line) finalizerQPanel.getComponent(0)).setGridSecundaryColor(secundaryColor);
            ((Line) compilePanel.getComponent(0)).setGridPrimaryColor(primaryColor);
            ((Line) compilePanel.getComponent(0)).setGridSecundaryColor(secundaryColor);
        }
    }

    private void changeGCSpacePanelColor(GCSpacePanel panel, Color color) {
        panel.getLine().setColor(color);
        panel.setForeground(color);
        BorderUtil.changeTitledBorderColor(panel, color);
    }

    private void changeLinePanelColor(JPanel panel, Color color) {
        ((Line) panel.getComponent(0)).setColor(color);
        BorderUtil.changeTitledBorderColor(panel, color);
    }
}