/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui;

import net.sf.intelliplugin.visualgc.util.Converter;
import net.sf.intelliplugin.visualgc.util.TextAreaRenderer;
import net.sf.intelliplugin.visualgc.model.GCSample;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.util.HashMap;
import java.util.Map;

/**
 * Panel containing information about a JVM.
 * 
 * @author Bart Cremers
 * @since 21-nov-2007
 */
public class VMInfoPanel extends JPanel {

    private static final String ELAPSED_TIME = "Elapsed Time";
    private static final String JAVA_COMMAND_LINE = "Java Command Line";
    private static final String JAVA_VM_ARGUMENTS = "Java VM Arguments";
    private static final String JAVA_VM_FLAGS = "Java VM Flags";
    private static final String JAVA_HOME = "java.home";
    private static final String JAVA_CLASS_PATH = "java.class.path";
    private static final String JAVA_LIBRARY_PATH = "java.library.path";
    private static final String JAVA_ENDORSED_DIRS = "java.endorsed.dirs";
    private static final String JAVA_EXT_DIRS = "java.ext.dirs";
    private static final String SUN_BOOT_CLASS_PATH = "sun.boot.class.path";
    private static final String SUN_BOOT_LIBRARY_PATH = "sun.boot.library.path";
    private static final String JAVA_VM_NAME = "java.vm.name";
    private static final String JAVA_VM_INFO = "java.vm.info";
    private static final String JAVA_VM_VENDOR = "java.vm.vendor";
    private static final String JAVA_VM_VERSION = "java.vm.version";
    private static final String JAVA_VM_SPECIFICATION_NAME = "java.vm.specification.name";
    private static final String JAVA_VM_SPECIFICATION_VENDOR = "java.vm.specification.vendor";
    private static final String JAVA_VM_SPECIFICATION_VERSION = "java.vm.specification.version";

    private static final String[] ROWS = {
        ELAPSED_TIME, JAVA_COMMAND_LINE, JAVA_VM_ARGUMENTS, JAVA_VM_FLAGS, JAVA_HOME, JAVA_CLASS_PATH,
        JAVA_LIBRARY_PATH, JAVA_ENDORSED_DIRS, JAVA_EXT_DIRS, SUN_BOOT_CLASS_PATH, SUN_BOOT_LIBRARY_PATH, JAVA_VM_NAME,
        JAVA_VM_INFO, JAVA_VM_VENDOR, JAVA_VM_VERSION, JAVA_VM_SPECIFICATION_NAME, JAVA_VM_SPECIFICATION_VENDOR,
        JAVA_VM_SPECIFICATION_VERSION
    };

    private final Map<String, String> values;
    private final VMInfoPanel.SampleDataModel dataModel;

    public VMInfoPanel(GCSample gcSample) {
        super(new BorderLayout());

        values = new HashMap<String, String>(ROWS.length);

        values.put(ELAPSED_TIME, Converter.longToTimeString(gcSample.getOsElapsedTime(), GCSample.getOsFrequency()));
        values.put(JAVA_COMMAND_LINE, GCSample.getJavaCommand());
        values.put(JAVA_VM_ARGUMENTS, GCSample.getVmArgs());
        values.put(JAVA_VM_FLAGS, GCSample.getVmFlags());
        values.put(JAVA_HOME, GCSample.getJavaHome());
        values.put(JAVA_CLASS_PATH, GCSample.getClassPath());
        values.put(JAVA_LIBRARY_PATH, GCSample.getLibraryPath());
        values.put(JAVA_ENDORSED_DIRS, GCSample.getEndorsedDirs());
        values.put(JAVA_EXT_DIRS, GCSample.getExtDirs());
        values.put(SUN_BOOT_CLASS_PATH, GCSample.getBootClassPath());
        values.put(SUN_BOOT_LIBRARY_PATH, GCSample.getBootLibraryPath());
        values.put(JAVA_VM_NAME, GCSample.getVmName());
        values.put(JAVA_VM_INFO, GCSample.getVmInfo());
        values.put(JAVA_VM_VENDOR, GCSample.getVmVendor());
        values.put(JAVA_VM_VERSION, GCSample.getVmVersion());
        values.put(JAVA_VM_SPECIFICATION_NAME, GCSample.getVmSpecName());
        values.put(JAVA_VM_SPECIFICATION_VENDOR, GCSample.getVmSpecVendor());
        values.put(JAVA_VM_SPECIFICATION_VERSION, GCSample.vetVmSpecVersion());

        dataModel = new SampleDataModel();
        JTable table = new JTable(dataModel);
        table.getColumnModel().getColumn(1).setCellRenderer(new TextAreaRenderer());
        add(new JScrollPane(table), BorderLayout.CENTER);
    }

    public void setSample(GCSample gcSample) {
        values.put(ELAPSED_TIME, Converter.longToTimeString(gcSample.getOsElapsedTime(), GCSample.getOsFrequency()));
        dataModel.fireTableCellUpdated(0, 1);
    }

    private class SampleDataModel extends AbstractTableModel {

        public int getRowCount() {
            return ROWS.length;
        }

        public int getColumnCount() {
            return 2;
        }

        public String getColumnName(int columnIndex) {
            switch (columnIndex) {
                case 0:
                    return "Field";
                case 1:
                    return "Value";
            }
            return null;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            switch (columnIndex) {
                case 0:
                    return ROWS[rowIndex];
                case 1:
                    return values.get(ROWS[rowIndex]);
            }
            return null;
        }

    }
}
