/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui;

import static net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration.*;
import net.sf.intelliplugin.visualgc.model.GCSample;
import net.sf.intelliplugin.visualgc.ui.graph.Level;
import net.sf.intelliplugin.visualgc.util.BorderUtil;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Panel displaying the JVM Heap.
 *
 * @author Bart Cremers
 * @since 23-nov-2007
 */
class VisualHeap extends JPanel implements ComponentListener, PropertyChangeListener {

    private final Level permLevel;
    private final Level tenuredLevel;
    private final Level edenLevel;
    private final Level s0Level;
    private final Level s1Level;

    private final JPanel permPanel;
    private final JPanel tenuredPanel;
    private final JPanel edenPanel;
    private final JPanel s0Panel;
    private final JPanel s1Panel;

    private GCSample currentSample;

    public VisualHeap(GCSample gcSample) {
        super(new GridBagLayout());

        currentSample = gcSample;

        permLevel = new Level(getConfiguration().getPermColor());
        tenuredLevel = new Level(getConfiguration().getTenuredColor());
        edenLevel = new Level(getConfiguration().getEdenColor());
        s0Level = new Level(getConfiguration().getSurvivorColor());
        s1Level = new Level(getConfiguration().getSurvivorColor());

        addComponentListener(this);

        edenPanel = createLevelPanel(getConfiguration().getEdenColor(), "Eden", edenLevel);
        s0Panel = createLevelPanel(getConfiguration().getSurvivorColor(), "S0", s0Level);
        s1Panel = createLevelPanel(getConfiguration().getSurvivorColor(), "S1", s1Level);
        tenuredPanel = createLevelPanel(getConfiguration().getTenuredColor(), "Tenured", tenuredLevel);
        permPanel = createLevelPanel(getConfiguration().getPermColor(), "Perm", permLevel);

        add(edenPanel);
        add(s0Panel);
        add(s1Panel);
        add(tenuredPanel);
        add(permPanel);

        resetPanel(gcSample);

        getConfiguration().addPropertyChangeListener(this);
    }

    private JPanel createLevelPanel(Color color, String borderText, Level level) {
        JPanel panel = new JPanel(new GridLayout(1, 1));
        panel.setBackground(getConfiguration().getBackgroundColor());
        panel.setBorder(BorderUtil.createTitledBorder(borderText, color));
        panel.addComponentListener(this);
        panel.add(level);

        panel.setPreferredSize(new Dimension(panel.getPreferredSize().width, 100));
        return panel;
    }

    public void componentResized(ComponentEvent e) {
        resetPanel(currentSample);
    }

    public void componentMoved(ComponentEvent e) {
    }

    public void componentShown(ComponentEvent e) {
    }

    public void componentHidden(ComponentEvent e) {
    }

    private void resetPanel(GCSample gcsample) {

        GridBagConstraints gridbagconstraints = new GridBagConstraints();
        gridbagconstraints.fill = GridBagConstraints.BOTH;
        gridbagconstraints.gridheight = GridBagConstraints.REMAINDER;
        gridbagconstraints.weighty = 1.0D;

        double totalHeapSize = gcsample.getPermSize() + gcsample.getTenuredSize() + gcsample.getNewGenMaxSize();
        double tenuredWeight = (double) gcsample.getTenuredSize() / totalHeapSize;
        double survivor0Weight = (double) gcsample.getSurvivor0Capacity() / totalHeapSize;
        double survivor1Weight = (double) gcsample.getSurvivor1Capacity() / totalHeapSize;
        double edenWeight = (gcsample.getNewGenMaxSize() - (survivor0Weight + survivor1Weight)) / totalHeapSize;
        double permWeight = 1.0D - (tenuredWeight + survivor0Weight + survivor1Weight + edenWeight);

        GridBagLayout layout = (GridBagLayout) getLayout();

        gridbagconstraints.weightx = edenWeight;
        layout.setConstraints(edenPanel, gridbagconstraints);

        gridbagconstraints.weightx = survivor0Weight;
        layout.setConstraints(s0Panel, gridbagconstraints);

        gridbagconstraints.weightx = survivor1Weight;
        layout.setConstraints(s1Panel, gridbagconstraints);

        gridbagconstraints.weightx = tenuredWeight;
        layout.setConstraints(tenuredPanel, gridbagconstraints);

        gridbagconstraints.weightx = permWeight;
        gridbagconstraints.gridwidth = GridBagConstraints.REMAINDER;
        layout.setConstraints(permPanel, gridbagconstraints);
    }

    public void update(GCSample gcSample) {
        if (gcSample.heapSizeChanged(currentSample)) {
            resetPanel(gcSample);
        }

        updateLevel(gcSample);

        repaint();

        currentSample = gcSample;
    }

    void updateLevel(GCSample gcsample) {
        permLevel.updateLevel(gcsample.getPermLiveRatio());
        permLevel.updateGrayLevel(1.0D - gcsample.getPermCommittedRatio());

        tenuredLevel.updateLevel(gcsample.getTenuredLiveRatio());
        tenuredLevel.updateGrayLevel(1.0D - gcsample.getTenuredCommittedRatio());

        edenLevel.updateGrayLevel(1.0D - (double) gcsample.getEdenCapacity() / gcsample.getAdjustedEdenSize());
        edenLevel.updateLevel((double) gcsample.getEdenUsed() / gcsample.getAdjustedEdenSize());

        s0Level.updateLevel((double) gcsample.getSurvivor0Used() / (double) gcsample.getSurvivor0Capacity());
        s1Level.updateLevel((double) gcsample.getSurvivor1Used() / (double) gcsample.getSurvivor1Capacity());
    }

    public void propertyChange(PropertyChangeEvent evt) {
        String pName = evt.getPropertyName();
        if (BACKGROUND_COLOR.equals(pName)) {
            Color bgColor = getConfiguration().getBackgroundColor();
            edenPanel.setBackground(bgColor);
            s0Panel.setBackground(bgColor);
            s1Panel.setBackground(bgColor);
            tenuredPanel.setBackground(bgColor);
            permPanel.setBackground(bgColor);
        } else if (PERM_COLOR.equals(pName)) {
            updateLevelPanel(permPanel, getConfiguration().getPermColor());
        } else if (TENURED_COLOR.equals(pName)) {
            updateLevelPanel(tenuredPanel, getConfiguration().getTenuredColor());
        } else if (EDEN_COLOR.equals(pName)) {
            updateLevelPanel(edenPanel, getConfiguration().getEdenColor());
        } else if (SURVIVOR_COLOR.equals(pName)) {
            updateLevelPanel(s0Panel, getConfiguration().getSurvivorColor());
            updateLevelPanel(s1Panel, getConfiguration().getSurvivorColor());
        } else if (BORDER_COLOR.equalsIgnoreCase(pName)) {
            BorderUtil.changeTitledBorderColor(permPanel, getConfiguration().getPermColor());
            BorderUtil.changeTitledBorderColor(tenuredPanel, getConfiguration().getTenuredColor());
            BorderUtil.changeTitledBorderColor(edenPanel, getConfiguration().getEdenColor());
            BorderUtil.changeTitledBorderColor(s0Panel, getConfiguration().getSurvivorColor());
            BorderUtil.changeTitledBorderColor(s1Panel, getConfiguration().getSurvivorColor());
        } else if (PRIMARY_GRID_COLOR.equalsIgnoreCase(pName) || SECUNDARY_GRID_COLOR.equalsIgnoreCase(pName)) {
            Color primaryColor = getConfiguration().getPrimaryGridColor();
            Color secundaryColor = getConfiguration().getSecundaryGridColor();
            permLevel.setGridColors(primaryColor, secundaryColor);
            tenuredLevel.setGridColors(primaryColor, secundaryColor);
            s0Level.setGridColors(primaryColor, secundaryColor);
            s1Level.setGridColors(primaryColor, secundaryColor);
            edenLevel.setGridColors(primaryColor, secundaryColor);
        }
    }

    private void updateLevelPanel(JPanel panel, Color color) {
        ((Level) panel.getComponent(0)).setColor(color);
        BorderUtil.changeTitledBorderColor(panel, color);
    }
}
