/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui.graph;

/**
 * @author Bart Cremers
 * @since 20-nov-2007
 */

import java.util.ArrayList;

/**
 * A list containing a maximum amount of numbers.
 */
public class FIFORangedList extends ArrayList<Number> {

    private int maxSize;
    private double maxValue;
    private double minValue;
    private boolean autoRange;

    /**
     * Create a new list with fixed minimum and maximum value.
     *
     * @param maxSize  The maximum size of the list.
     * @param minValue A minimum value for the list.
     * @param maxValue A maximum value for the list.
     */
    public FIFORangedList(int maxSize, double minValue, double maxValue) {
        super(maxSize);
        this.maxSize = maxSize;
        this.minValue = minValue;
        this.maxValue = maxValue;
        autoRange = false;
    }

    /**
     * Create a new list which will automatically calculate its range.
     *
     * @param maxSize The maximum size of the list.
     */
    public FIFORangedList(int maxSize) {
        this(maxSize, Double.MAX_VALUE, Double.MIN_VALUE);
        this.autoRange = true;
    }

    /**
     * Add a number to the list. This possibly removes the oldest number in the list.
     *
     * @param number the number to add
     * @return <code>true</code> if this collection changed as a result of the call
     */
    public boolean add(Number number) {
        Number current = null;

        if (size() > maxSize) {
            current = get(0);
            super.remove(0);
        }

        boolean flag = super.add(number);

        if (autoRange) {
            if (current == null) {
                recomputeRange(number.doubleValue());
            } else {
                recomputeRange(current.doubleValue(), number.doubleValue());
            }
        }

        return flag;
    }

    private void recomputeRange(double value) {
        if (value > maxValue) {
            maxValue = value;
        } else if (value < minValue) {
            minValue = value;
        }
    }

    private void recomputeRange(double maxValue, double minValue) {
        this.maxValue = recomputeMaxValue(maxValue, minValue);
        this.minValue = recomputeMinValue(maxValue, minValue);
    }

    private double recomputeMaxValue(double maxValue, double minValue) {
        if (minValue >= this.maxValue) {
            return minValue;
        }

        if (maxValue == this.maxValue) {
            return findMaxValue();
        } else {
            return this.maxValue;
        }
    }

    private double recomputeMinValue(double maxValue, double minValue) {
        if (minValue <= this.minValue) {
            return minValue;
        }

        if (maxValue == this.minValue) {
            return findMinValue();
        } else {
            return this.minValue;
        }
    }

    private double findMinValue() {
        double minValue = Double.MAX_VALUE;

        for (Number number : this) {
            double d1 = number.doubleValue();

            if (d1 < minValue) {
                minValue = d1;
            }
        }

        return minValue;
    }

    private double findMaxValue() {
        double minValue = Double.MIN_VALUE;

        for (Number number : this) {
            double d1 = number.doubleValue();

            if (d1 > minValue) {
                minValue = d1;
            }
        }

        return minValue;
    }

    /**
     * @return <code>true</code> if the list automatically recalculates its range.
     */
    public boolean isAutoRange() {
        return autoRange;
    }

    /**
     * @param flag Sets the autorange feature of the list.
     * @see #isAutoRange()
     */
    public void setAutoRange(boolean flag) {
        if (!autoRange && flag) {
            maxValue = findMaxValue();
            minValue = findMinValue();
        }

        autoRange = flag;
    }

    /**
     * @return The maximum value in the list.
     */
    public double getMaxValue() {
        return maxValue;
    }

    /**
     * @return The minimum value in the list.
     */
    public double getMinValue() {
        return minValue;
    }

    /**
     * Sets a specific maximum value for the list.
     * @param maxValue the new maximum value
     * @throws IllegalStateException if the list is currently in autorange mode
     */
    public void setMaxValue(double maxValue) {
        if (autoRange) {
            throw new IllegalStateException();
        } else {
            this.maxValue = maxValue;
        }
    }

    /**
     * Sets a specific minimum value for the list.
     * @param minValue the new minimum value
     * @throws IllegalStateException if the list is currently in autorange mode
     */
    public void setMinValue(double minValue) {
        if (autoRange) {
            throw new IllegalStateException();
        } else {
            this.minValue = minValue;
        }
    }

    public Object clone() {
        FIFORangedList fifolist = (FIFORangedList) super.clone();

        fifolist.maxSize = maxSize;
        fifolist.maxValue = maxValue;
        fifolist.minValue = minValue;
        fifolist.autoRange = autoRange;

        return fifolist;
    }
}