/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.ui.graph;

import net.sf.intelliplugin.visualgc.config.IntelliHeapConfiguration;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

/**
 * A component painting a graph over time on a grid.
 *
 * @author Bart Cremers
 * @since 21-nov-2007
 */
public class Line extends JComponent {

    private static final int DEFAULT_DATASET_SIZE = 1000;
    private static final int XSCALE = 2;
    private Color color;
    private final GridDrawer gridDrawer;
    private final FIFORangedList dataset;

    public Line(Color color1) {
        this(new FIFORangedList(DEFAULT_DATASET_SIZE), color1);
    }

    public Line(FIFORangedList fifolist) {
        this(fifolist, Color.LIGHT_GRAY);
    }

    public Line(FIFORangedList fifolist, Color color) {
        dataset = fifolist;
        this.color = color;
        gridDrawer = new GridDrawer(10, 10, IntelliHeapConfiguration.getConfiguration().getPrimaryGridColor());
        gridDrawer.setSecundaryColor(IntelliHeapConfiguration.getConfiguration().getSecundaryGridColor());
        setPreferredSize(new Dimension(600, 100));
        addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent componentevent) {
                super.componentResized(componentevent);
                validate();
            }
        });
    }

    public Color getColor() {
        return color;
    }

    public void setColor(Color color) {
        this.color = color;
        repaint();
    }

    public Color getGridPrimaryColor() {
        return gridDrawer.getPrimaryColor();
    }

    public void setGridPrimaryColor(Color color1) {
        gridDrawer.setPrimaryColor(color1);
        repaint();
    }

    public Color getGridSecundaryColor() {
        return gridDrawer.getSecundaryColor();
    }

    public void setGridSecundaryColor(Color color1) {
        gridDrawer.setSecundaryColor(color1);
        repaint();
    }

    void drawGraph(Graphics g) {
        if (dataset.isEmpty()) {
            return;
        }
        int width = getWidth();
        int height = getHeight();
        double maxValue = dataset.getMaxValue();
        int datasetSize = dataset.size();
        int nPoints = Math.min(datasetSize, width / XSCALE);
        double pixelToValue = (double) height / maxValue;
        int[] xPoints = new int[nPoints + 2];
        int[] yPoints = new int[nPoints + 2];
        int xStart = width - nPoints * 2;
        xPoints[0] = xStart;
        yPoints[0] = height;
        for (int point = 0; point < nPoints; point++) {
            double currentValue = dataset.get(datasetSize - nPoints + point).doubleValue();
            int pointHeight = height - (int) (currentValue * pixelToValue);
            if (currentValue > 0.0D && pointHeight < 0) {
                pointHeight = 0;
            }
            if (currentValue > 0.0D && pointHeight >= height) {
                pointHeight = height - 1;
            }
            xPoints[point + 1] = xStart + point * 2;
            yPoints[point + 1] = pointHeight;
        }

        xPoints[nPoints + 1] = width;
        yPoints[nPoints + 1] = height;
        g.setColor(color);
        g.fillPolygon(xPoints, yPoints, nPoints + 2);
    }

    public void updateGrayLevel(double d) {
        gridDrawer.splitRange(d);
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        gridDrawer.draw(g, getWidth(), getHeight());
        drawGraph(g);
    }
}