/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.util;

import java.text.NumberFormat;

/**
 * Helper class for converting units.
 * 
 * @author Bart Cremers
 * @since 21-nov-2007
 */
public class Converter {
    private static final long ONE_GB = 1073741824L;
    private static final long ONE_MB = 1048576L;
    private static final long ONE_KB = 1024L;

    private static final double NANO_DIVIDER = 1000000000D;
    private static final double MICRO_DIVIDER = 1000000D;
    private static final double MILLI_DIVIDER = 1000D;
    private static final double MINUTE_HOUR_MULTIPLIER = 60D;
    private static final double DAY_MULTIPLIER = 24D;

    private static final NumberFormat nf;

    static {
        nf = NumberFormat.getNumberInstance();
        nf.setMaximumFractionDigits(3);
        nf.setMinimumFractionDigits(3);
        nf.setMinimumIntegerDigits(1);
    }
    
    private Converter() {
    }

    public static String longToKMGString(Number number) {
        return longToKMGString(number.longValue());
    }

    public static String longToKMGString(long bytes) {
        float value = (float) bytes / ONE_GB;
        if ((long) value > 0L) {
            return nf.format(value) + "G";
        }
        value = (float) bytes / ONE_MB;
        if ((long) value > 0L) {
            return nf.format(value) + "M";
        }
        value = (float) bytes / ONE_KB;
        if ((long) value > 0L) {
            return nf.format(value) + "K";
        } else {
            return String.valueOf(bytes);
        }
    }

    public static String longToTimeString(Number elapsedtime, Number frequency) {
        return longToTimeString(elapsedtime.longValue(), frequency.longValue());
    }

    public static String longToTimeString(long elapsedTime, long frequency) {
        double ticksPerNano = (double) frequency / NANO_DIVIDER;
        double ticksPerMicro = (double) frequency / MICRO_DIVIDER;
        double ticksPerMilli = (double) frequency / MILLI_DIVIDER;
        double ticksPerMinute = (double) frequency * MINUTE_HOUR_MULTIPLIER;
        double ticksPerHour = ticksPerMinute * MINUTE_HOUR_MULTIPLIER;
        double ticksPerDay = ticksPerHour * DAY_MULTIPLIER;

        if (elapsedTime == 0L) {
            return "0s";
        }

        StringBuilder sb = new StringBuilder();
        boolean dontShowMillis = false;

        double time = elapsedTime;
        double ticks = time / ticksPerDay;
        long ticksToGo = (long) ticks;
        double d9 = time - (double) ticksToGo * ticksPerDay;
        if (ticksToGo > 0L || dontShowMillis) {
            sb.append(ticksToGo).append("d ");
            dontShowMillis = true;
            if (ticksToGo > 0L) {
                time = d9;
            }
        }
        ticks = time / ticksPerHour;
        ticksToGo = (long) ticks;
        d9 = time - (double) ticksToGo * ticksPerHour;
        if (ticksToGo > 0L || dontShowMillis) {
            sb.append(ticksToGo).append("h ");
            dontShowMillis = true;
            if (ticksToGo > 0L) {
                time = d9;
            }
        }
        ticks = time / ticksPerMinute;
        ticksToGo = (long) ticks;
        d9 = time - (double) ticksToGo * ticksPerMinute;
        if (ticksToGo > 0L || dontShowMillis) {
            sb.append(ticksToGo).append("m ");
            dontShowMillis = true;
            if (ticksToGo > 0L) {
                time = d9;
            }
        }
        ticks = time / (double) frequency;
        ticksToGo = (long) ticks;
        if (ticksToGo > 0L || dontShowMillis) {
            sb.append(nf.format(ticks)).append("s ");
            dontShowMillis = true;
        }
        if (dontShowMillis) {
            return sb.toString();
        }


        ticks = (double) elapsedTime / ticksPerMilli;
        if (ticks >= 1.0D) {
            return nf.format(ticks) + "ms";
        }
        ticks = (double) elapsedTime / ticksPerMicro;
        if (ticks >= 1.0D) {
            return nf.format(ticks) + "us";
        } else {
            double nanos = (double) elapsedTime / ticksPerNano;
            return nf.format(nanos) + "ns";
        }
    }
}