/***************************************************************************
 * Copyright (c) 2007, Bart Cremers                                        *
 * All rights reserved.                                                    *
 *                                                                         *
 * Redistribution and  use in source  and binary  forms,  with or  without *
 * modification,  are permitted provided that the following conditions are *
 * met:                                                                    *
 *                                                                         *
 *   - Redistributions  of source  code  must retain  the above  copyright *
 *     notice, this list of conditions and the following disclaimer.       *
 *   - Redistributions in binary  form must reproduce  the above copyright *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation   and/or   other    materials   provided   with   the *
 *     distribution.                                                       *
 *   - Neither  the  name  of  the  organization  nor  the  names  of  its *
 *     contributors  may be  used to  endorse or  promote products derived *
 *     from this software without specific prior written permission.       *
 *                                                                         *
 * THIS SOFTWARE  IS PROVIDED  BY THE COPYRIGHT  HOLDERS AND  CONTRIBUTORS *
 * "AS IS"  AND ANY  EXPRESS OR  IMPLIED WARRANTIES,  INCLUDING,  BUT  NOT *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR  PURPOSE  ARE DISCLAIMED.  IN NO  EVENT SHALL  THE COPYRIGHT *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,  INDIRECT,  INCIDENTAL, *
 * SPECIAL,  EXEMPLARY,  OR  CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT *
 * LIMITED TO,  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, *
 * DATA, OR PROFITS;  OR BUSINESS INTERRUPTION)  HOWEVER CAUSED AND ON ANY *
 * THEORY OF LIABILITY,  WHETHER IN CONTRACT,  STRICT  LIABILITY,  OR TORT *
 * (INCLUDING  NEGLIGENCE OR OTHERWISE)  ARISING IN ANY WAY OUT OF THE USE *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/
package net.sf.intelliplugin.visualgc.util;

import org.jdom.Element;
import org.jdom.DataConversionException;

import java.awt.*;
import java.util.*;
import java.util.List;

/**
 * Wrapper around {@link com.intellij.openapi.util.JDOMExternalizer} adding functionality to write colors and lists.
 * 
 * @author Bart Cremers
 * @since 4-dec-2007
 */
public final class JDOMExternalizer {

    private JDOMExternalizer() {
    }

    public static Color readColor(Element root, String name, Color defaultValue) {
        String colorValue = readString(root, name);
        if (colorValue != null) {
            try {
                return new Color(Integer.parseInt(colorValue, 16));
            } catch (NumberFormatException e) {
                // Ignore
            }
        }
        return defaultValue;
    }

    public static boolean readBoolean(Element root, String name) {
        return com.intellij.openapi.util.JDOMExternalizer.readBoolean(root, name);
    }

    public static int readInteger(Element root, String name, int defaultValue) {
        return com.intellij.openapi.util.JDOMExternalizer.readInteger(root, name, defaultValue);
    }

    public static String readString(Element root, String name) {
        return com.intellij.openapi.util.JDOMExternalizer.readString(root, name);
    }

    public static void readMap(Element root, Map<String, String> map, String name, String value) {
        com.intellij.openapi.util.JDOMExternalizer.readMap(root, map, name, value);
    }

    public static void write(Element root, String name, boolean value) {
        com.intellij.openapi.util.JDOMExternalizer.write(root, name, value);
    }

    public static void write(Element root, String name, int value) {
        com.intellij.openapi.util.JDOMExternalizer.write(root, name, value);
    }

    public static void write(Element root, String name, String value) {
        com.intellij.openapi.util.JDOMExternalizer.write(root, name, value);
    }

    public static void write(Element root, String name, Color value) {
        if (value != null) {
            write(root, name, Integer.toString(value.getRGB() & 0xFFFFFF, 16));
        }
    }

    public static void writeMap(Element root, Map<String, String> map, String name, String value) {
        com.intellij.openapi.util.JDOMExternalizer.writeMap(root, map, name, value);
    }

    public static void writeList(Element root, List<String> list) {
        for (int i = 0; i < list.size(); i++) {
            Element element = new Element("item");
            element.setAttribute("index", String.valueOf(i));
            element.setAttribute("value", list.get(i));
            root.addContent(element);
        }
    }

    @SuppressWarnings ({ "unchecked" })
    public static List<String> readList(Element root) {
        SortedSet<ListItem> sortedItems = new TreeSet<ListItem>();

        List<Element> children = root.getChildren("item");

        for (Element element : children) {
            try {
                int index = element.getAttribute("index").getIntValue();
                String value = element.getAttribute("value").getValue();
                sortedItems.add(new ListItem(index, value));
            } catch (DataConversionException e) {
                // Ignore
            }
        }

        List<String> list = new ArrayList<String>(sortedItems.size());

        for (ListItem item : sortedItems) {
            list.add(item.getValue());
        }

        return list;
    }

    private static class ListItem implements Comparable<ListItem> {
        private final int index;
        private final String value;

        private ListItem(int index, String value) {
            this.index = index;
            this.value = value;
        }

        public String getValue() {
            return value;
        }

        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (o == null || getClass() != o.getClass()) {
                return false;
            }

            ListItem listItem = (ListItem) o;

            return index == listItem.index;
        }

        public int hashCode() {
            return index;
        }

        public int compareTo(ListItem o) {
            return o.index - this.index;
        }
    }
}
